package com.alterdekim.hearthhack.component;

import com.alterdekim.hearthhack.component.interfaces.GamePacketCallback;
import com.alterdekim.hearthhack.component.interfaces.IGameRoom;
import com.alterdekim.hearthhack.dto.RoomPlayerDTO;
import com.alterdekim.hearthhack.game.GameState;
import com.alterdekim.hearthhack.game.IncomeGamePacket;
import com.alterdekim.hearthhack.game.OutcomeGamePacket;
import com.alterdekim.hearthhack.service.UserService;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.stream.Collectors;

@Slf4j
public class GameRoom extends Thread implements IGameRoom {
    @Getter
    private final List<RoomPlayerDTO> players;

    private final ConcurrentHashMap<Long, GamePacketCallback> callbacks;

    private final UserService userService;

    @Getter
    private final String password;

    private GameState globalState;

    @Getter
    private ConcurrentLinkedQueue<IncomeGamePacket> incomeQueue;
    private ConcurrentLinkedQueue<OutcomeGamePacket> outcomeQueue;

    public GameRoom(List<RoomPlayerDTO> players, UserService userService, String password) {
        this.players = players;
        this.userService = userService;
        this.password = password;
        this.incomeQueue = new ConcurrentLinkedQueue<>();
        this.outcomeQueue = new ConcurrentLinkedQueue<>();
        this.callbacks = new ConcurrentHashMap<>();
        this.globalState = GameState.CreateGame;
        this.start();
    }

    private void processIncomePackets() {
        if( this.incomeQueue.isEmpty() ) return;
        IncomeGamePacket packet = this.incomeQueue.peek();
        if( findPlayerById(packet.getPlayerId()).isEmpty() ) return;
        processIncomePacket(packet);
        this.incomeQueue.poll();
    }

    private void processOutcomePackets() {
       /* if( this.outcomeQueue.isEmpty() ) return;
        OutcomeGamePacket packet = this.outcomeQueue.peek();
        this.outcomeQueue.poll();*/
    }

    private void processIncomePacket(IncomeGamePacket packet) {
        if( packet.getPacket().getType() != 1) return;
        List<OutcomeGamePacket> l = this.outcomeQueue.stream()
                .filter(p -> p.getPlayerId().longValue() == packet.getPlayerId().longValue())
                .collect(Collectors.toList());
        GamePacketCallback callback = this.callbacks.get(packet.getPlayerId());
        if (callback == null) return;
        l.forEach(p -> callback.onMessage(p.getPacket()));
    }

    private void updateGameState() {
        // giant switch
        switch (this.globalState) {
            case CreateGame -> createGameEntity();
            case CreatePlayers -> createPlayerEntity();
            case CreateDecks -> createDeckEntities();
            case CreateHeroes -> createPlayerHeroEntity();
            case CreateHeroPowers -> createPlayerHeroPowerEntity();
        }
    }

    public void updateCallback(Long playerId, GamePacketCallback callback) {
        this.callbacks.put(playerId, callback);
    }

    @Override
    public void run() {
        while(true) {
            processIncomePackets();
            updateGameState();
            processOutcomePackets();
        }
    }

    @Override
    public void createGameEntity() {

    }

    @Override
    public void createPlayerEntity() {

    }

    @Override
    public void createDeckEntities() {

    }

    @Override
    public void createCardEntity() {

    }

    @Override
    public void createPlayerHeroEntity() {

    }

    @Override
    public void createPlayerHeroPowerEntity() {

    }

    private Optional<RoomPlayerDTO> findPlayerById(Long id) {
        return players.stream().filter(p -> p.getUserId().longValue() == id.longValue()).findFirst();
    }
}
